'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ZormErrors = require('./ZormErrors');

var _ZormErrors2 = _interopRequireDefault(_ZormErrors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Zorm = function () {
    /**
     * Create a new form object.
     */
    function Zorm(data) {
        _classCallCheck(this, Zorm);

        $.extend(true, this, data);

        this.busy = false;
        this.successful = false;
        this.originalData = data;
        this.errors = new _ZormErrors2.default();
    }

    /**
     * Get the data payload for the form.
     */


    _createClass(Zorm, [{
        key: 'payload',
        value: function payload() {
            return _.omit(JSON.parse(JSON.stringify(this)), ['busy', 'successful', 'errors', 'originalData']);
        }

        /**
         * Start processing the form.
         */

    }, {
        key: 'startProcessing',
        value: function startProcessing() {
            this.errors.forget();

            this.busy = true;
            this.successful = false;
        }

        /**
         * Finish processing the form.
         */

    }, {
        key: 'finishProcessing',
        value: function finishProcessing() {
            this.busy = false;
            this.successful = true;
        }

        /**
         * Finish processing the form with the given errors.
         */

    }, {
        key: 'finishProcessingWithErrors',
        value: function finishProcessingWithErrors(errors) {
            this.errors.set(errors);

            this.busy = false;
        }

        /**
         * Reset the form back to its original state.
         */

    }, {
        key: 'reset',
        value: function reset() {
            $.extend(true, this, this.originalData);
        }

        /**
         * Helper method for making POST HTTP requests.
         */

    }, {
        key: 'post',
        value: function post(uri) {
            return this.send('post', uri);
        }

        /**
         * Helper method for making PUT HTTP requests.
         */

    }, {
        key: 'put',
        value: function put(uri) {
            return this.send('put', uri);
        }

        /**
         * Helper method for making DELETE HTTP requests.
         */

    }, {
        key: 'delete',
        value: function _delete(uri) {
            return this.send('delete', uri);
        }

        /**
         * Send the form to the back-end server.
         */

    }, {
        key: 'send',
        value: function send(method, uri) {
            var _this = this;

            return new Promise(function (resolve, reject) {
                _this.startProcessing();

                Vue.http[method](uri, _this.payload()).then(function (response) {
                    _this.finishProcessing();

                    resolve(response.data);
                }).catch(function (errors) {
                    _this.finishProcessingWithErrors(errors.data);

                    reject(errors.data);
                });
            });
        }
    }]);

    return Zorm;
}();

exports.default = Zorm;